
import pygame
from pygame.locals import *
from pygext.gl.all import *
from tilemap import TileMap
import random, math, threading, time

import pySonic

import pylons
from pygext.render.stencils import alpha_stencil

def preload_graphics():
    for i in range(1,6):
        resources.get_bitmap("gfx/newtile%i.png" % i).set_hotspot(0.5, 0.5)
    s = pygame.image.load("gfx/newtile3.png")
    s2 = alpha_stencil(s)
    resources.init_bitmap("tile_stencil", s2).set_hotspot(0.5, 0.5)
        
    pylons.init_pylons()

    anim = []
    for i in range(1,4):
        f = Bitmap("gfx/lightning%i.png" % i).set_hotspot(0.84,0.825).scale(0.605)
        anim.append(f)
    resources.init_anim("lightning", anim)

    for i in range(1,5):
        resources.get_bitmap("gfx/background%i.png" % i)

    resources.get_bitmap("gfx/ui_left.png")
    resources.get_bitmap("gfx/ui_right.png")
    resources.get_bitmap("gfx/ui_bottom.png")

    resources.get_bitmap("gfx/intro_opioid.png")
    resources.get_bitmap("gfx/intro_pylonoid_a.png")
    resources.get_bitmap("gfx/an.png")
    resources.get_bitmap("gfx/game.png")

class Game(object):
    def __init__(self):
        self.font = GLFont(("fonts/GoudySans-Regular.ttf", 36))
        self.fader = rect(0,0,800,600).fillcolor(InheritColor)
        self.score = 0
        self.music_src = pySonic.Source()
        self.level = 0

    def init_background(self):
        self.bg = Background()

class Background(object):
    def __init__(self):
        i = random.randrange(1,5)
        bmp = Bitmap("gfx/background%i.png" % i).scale(4)
        self.bg = Entity(bmp) #.place("bg")
        self.bg.x = -random.randrange(500)-500
        self.bg.y = -random.randrange(500)-500
        self.bg_mover = EntityMover(self.bg)
        d = random.random()*2*math.pi
        self.bg_mover.vx = math.cos(d)*30
        self.bg_mover.vy = math.sin(d)*30      

        self.bg2 = Entity() #.place("bg2")
        self.bg2_mover = EntityMover(self.bg2)

        d = random.random()*2*math.pi
        self.bg2_mover.vx = math.cos(d)*30
        self.bg2_mover.vy = math.sin(d)*30

        self.bg_swap = 1000
        self.colorfade()

    def rand_color(self):
        c = random.randrange(5)
        if c == 0:
            r,g,b = 1,0.2,0.2
        elif c == 1:
            r,g,b = 0.2,1,0.2
        elif c == 2:
            r,g,b = 1,1,0.2
        elif c == 3:
            r,g,b = 0.2,1,1
        elif c == 4:
            r,g,b = 0.2,0.2,1
        r *= random.randrange(190,250)
        g *= random.randrange(190,250)
        b *= random.randrange(190,250)
        return int(r),int(g),int(b)
        

    def colorfade(self):
        r,g,b = self.rand_color()
        dcol = (r,g,b,255)
        self.bg.do(ColorFade(dcol, secs=5))

    def place(self):
        self.bg.place("bg")
        self.bg2.place("bg2")        

    def tick(self):
        self.bg_mover.tick()
        self.bg2_mover.tick()
        if not director.ticker.realtick:
            return

        for bg,bgm in ((self.bg,self.bg_mover),(self.bg2, self.bg2_mover)):
            if (bg.x < -512*4+820 and bgm.vx < 0) or (bg.x > -20 and bgm.vx > 0):
                bgm.vx = -bgm.vx
            if (bg.y < -512*4+620 and bgm.vy < 0) or (bg.y > -20 and bgm.vy > 0):
                bgm.vy = -bgm.vy

        bg = self.bg
        bgm = self.bg_mover
        if not bg.current_actions and random.randrange(500) == 0:
            self.colorfade()
            

##        if random.randrange(2000) == 0:
##            d = random.random()*2*math.pi
##            bgm.vx = math.cos(d)*30
##            bgm.vy = math.sin(d)*30

        self.bg_swap -= 1
        if self.bg_swap <= 0 and not bg.current_actions:
            self.bg_swap = 1000
            #self.bg.abort_actions()
            self.bg2.abort_actions()
            
            self.bg,self.bg2 = self.bg2,self.bg
            self.bg_mover,self.bg2_mover = self.bg2_mover,self.bg_mover

            self.bg.place("bg")
            self.bg2.place("bg2")
            
            dst = self.bg
            i = random.randrange(1,5)
            bmp = resources.get_bitmap("gfx/background%i.png" % i).scale(4)
            dst.shape = bmp
            dst.x = -random.randrange(500)-500
            dst.y = -random.randrange(500)-500
            d = random.random()*2*math.pi
            self.bg_mover.vx = math.cos(d)*30
            self.bg_mover.vy = math.sin(d)*30
            
            r,g,b = self.rand_color()
            dst.color = (r,g,b,0)
            act = ColorFade((r,g,b,255), secs=10)
            dst.do(act)

class InitScene(Scene):

    def enter(self, game):
        pygame.mixer.quit()
        game.sfx_world = pySonic.World()
        self.game = game

        self.new_layer("text")

        title = TextEntity(game.font, "Loading....", scale=1.5)
        title.centerx = 400
        title.centery = 300
        title.place("text")

        self.title = title
        self.loaded = False

    def tick(self):
        if not self.loaded:
            preload_graphics()
            self.loaded = True
            act = ColorFade((255,255,255,0), secs = 0.5) + SetScene(IntroScene, self.game)
            self.title.do(act)

class IntroScene(Scene):
    def enter(self, game):
        self.game = game

        self.new_layer("intro")
        
        game.music_src.Sound = pySonic.FileStream("sfx/cutemenu.mp3")
        game.music_src.LoopMode = pySonic.Constants.FSOUND_LOOP_NORMAL
        game.music_src.Play()

        opioid = Entity("gfx/intro_opioid.png")
        opioid.place("intro").set(centerx=400,centery=300)
        opioid.color = (255,255,255,0)
        opioid.do(ColorFade((255,255,255,255), secs=2))

        self.opioid = opioid
        
        act = Delay(3.5) + CallFunc(self.flash_texts)
        act.do()

    def flash_texts(self):
        an = Entity("gfx/an.png")
        game = Entity("gfx/game.png")
        o = self.opioid
        
        an.place("intro").set(centerx=o.centerx-100, bottom=o.top)
        game.place("intro").set(centerx=o.centerx+100, top=o.bottom)

        act = ColorFade((255,255,255,255), secs=0.3) + ColorFade((255,255,255,0), secs=0.3)
        for e in (an,game):
            e.color = (255,255,255,0)

        act.do(an)
        game.do(Delay(0.6) + act)
        o.do(Delay(1.0) + ColorFade((255,255,255,0), secs=1))

        pylonoid = Entity("gfx/intro_pylonoid_a.png")
        pylonoid.place("intro").set(centerx=400, centery=300)
        pylonoid.color = (255,255,255,0)
        pylonoid.do(Delay(2) + ColorFade((255,255,255,255), secs=2))
        (Delay(4) + SetScene(TitleScene, self.game)).do()

    def handle_mousebuttondown(self, ev):
        director.set_scene(TitleScene, self.game)

title_texts = [
    "Team Opioid presents",
    "a Pyweek challenge entry",
    "Visuals and music by\n         Petteri Liimatainen",
    "Programming by\n       Sami Hangaslammi",
    "Instructions:",
    "Use the arrow keys to\n       scroll around the map",
    "The flashing pylon\n      is the source of power",
    "Electicity spreads to the direction of the arrows\n                 But never against an arrow",
    "The electring charge is released when\n          your time runs out",
    "or if you right click\n      on the flashing pylon",
    "Once the target is hit\n      no more tiles are electrified",
    "Good luck!",
    ]

class TitleScene(Scene):
    def enter(self, game):
        self.game = game
        self.new_layer("bg2", -10)
        self.new_layer("bg", -5)
        self.new_layer("title", 10)
        self.new_layer("fade", 0)
        
        self.game.init_background()
        self.game.bg.place()

        font = self.game.font
        e = Entity("gfx/intro_pylonoid_a.png")
        e.place("title").set(centerx=400, centery=300)
        MoveDelta(0, -180, secs=3.0).do(e)

        fader = Entity(game.fader).place("fade")
        fader.color = (0,0,0,255)
        act = ColorFade((0,0,0,0), secs=1) + Delete
        fader.do(act)

        txt = TextEntity(self.game.font, "Click mouse to begin\nor wait for instructions", scale=0.7)
        txt.place("title").set(centerx=400, bottom=600)
        txt.color = (255,255,255,0)
        txt.do(Delay(2) + ColorFade((255,255,255,255), secs=1))

        self.state = "title"
        self.text_index = 0

        (Delay(3) + CallFunc(self.flash_next_text)).do()

        self.new_game()


    def tick(self):
        self.game.bg.tick()

    def state_title_handle_mousebuttondown(self, ev):
        self.state = "fading"
        self.game.music_src.Stop()
        act = ColorFade((255,255,255,0), secs=0.5)
        for e in self["title"]:
            act.do(e)
        (Delay(0.5)+SetScene(MainScene, self.game)).do()

    def flash_next_text(self):
        if self.text_index >= len(title_texts):
            return
        e = TextEntity(self.game.font, title_texts[self.text_index])
        self.text_index += 1
        e2 = Entity(e.shape)
        e3 = Entity(e.shape)
        act = Delay(3) + ColorFade((255,255,255,0), secs=0.3) + Delete
        (Delay(5) + CallFunc(self.flash_next_text)).do()        
        for x in (e,e2,e3):
            x.color = (255,255,255,30)
            x.place("title").set(centerx=400, centery=300)
            x.do(act)
        e.color = (255,255,255,150)
        #e.do(MoveDelta(-5,-5, secs=0.02, mode=PingPongMode))
        e2.x -= 5
        e2.y -= 5
        e2.do(MoveDelta(10, 10, secs=0.3, mode=PingPongMode))
        e3.x += 5
        e3.y += 5        
        e3.do(MoveDelta(-10, -10, secs=0.45, mode=PingPongMode))

    def new_game(self):
        self.game.level = 0
        self.game.score = 0

class Level(object):
    def __init__(self, goal, time, size):
        self.goal = goal
        self.time = time
        self.size = size

levels = [
    Level(goal=5, time=20, size=4),
    Level(goal=8, time=15, size=5),
    Level(goal=10, time=15, size=6),
    Level(goal=15, time=15, size=6),
    Level(goal=20, time=15, size=7),
    Level(goal=25, time=12, size=7),
    Level(goal=30, time=12, size=8),
    Level(goal=40, time=12, size=8),
    Level(goal=50, time=10, size=9),
    Level(goal=60, time=10, size=9),
    Level(goal=70, time=10, size=10),
    Level(goal=85, time=10, size=11),
    Level(goal=100, time=10, size=12),
    Level(goal=125, time=10, size=13),
    Level(goal=150, time=10, size=14),
    Level(goal=200, time=10, size=15),
    Level(goal=300, time=10, size=16),
    ]
            
class MainScene(Scene):

    def enter(self, game):
        self.game = game
        
        self.init_layers()
        self.init_background()
        self.init_ui()
        self.init_score()
        self.init_music()

        fader = Entity(game.fader).place("fade")
        fader.color = (0,0,0,100)
        fader.hidden = True
        self.fader = fader

        self.next_level()


    def next_level(self):
        director.reactor.clear()
        game = self.game
        self.level = levels[game.level]
        game.level += 1
        self.init_map()
        self.start_timer = self.init_timer(20)

        self.state = "intro"
        self.show_intro()

    def init_timer(self, time=20):
        r = rect(0,0,9,10).fillcolor(InheritColor)
        timers = []
        self.timers = timers
        for i in range(time):
            e = Entity(r)
            e.color = (0,100,200,255)
            e.x = 777
            e.y = 442-i*12
            e.place("ui")
            timers.append(e)
        act = ColorFade((0,100,200,0), secs=5)
        def fade(i=len(timers)-1):
            if i > 0:
                act.do(timers[i]).chain(CallFunc(fade, i-1) + Delete)
            else:
                act.do(timers[i]).chain(CallFunc(self.timer_end) + Delete)
        return fade
            

    def init_score(self):
        s = self.score_label = TextEntity(self.game.font, "000000")
        s.place("ui").set(right=120, bottom=600-20)

    def update_score(self, add):
        self.game.score += add
        self.score_label.set_text("%06i" % self.game.score)
        self.score_label.right = 120

    def init_layers(self):
        self.new_layer("bg2", -10)
        self.new_layer("bg", -5)
        self.new_layer("tiles", 0)
        self.new_layer("static_ui", 5)
        self.new_layer("ui", 10)
        self.new_layer("intro", 15)
        self.new_layer("fade", 20)
        self.new_layer("extro", 25)

    def init_music(self):
        game = self.game
        game.music_src.Sound = pySonic.FileStream("sfx/cuteforcer.mp3")
        game.music_src.LoopMode = pySonic.Constants.FSOUND_LOOP_NORMAL
        #game.music_src.SetEndStreamCallback(self.music_loop)
        game.music_src.Play()
        
    def music_loop(self, src):
        music = self.game.music_src
        print "stopping"
        music.Stop()
        print "new source"
        music = self.game.music_src = pySonic.Source()
        print "opening stream"
        music.Sound = pySonic.FileStream("sfx/cuteloop.mp3")
        print "setting loopmode"
        music.LoopMode = pySonic.Constants.FSOUND_LOOP_NORMAL
        print "playing"
        music.Play()
        print "OK"

    def init_map(self):        
        self.tilemap = TileMap(size=self.level.size)
        self.tilemap.place("tiles").set(x=400,y=300)
        self.zoomlevel = 0
        self.tilemap.raise_pieces()

    def init_background(self):
        self.game.bg.place()

    def init_ui(self):
        ui_left = Entity("gfx/ui_left.png").place("static_ui")
        ui_bottom = Entity("gfx/ui_bottom.png").place("static_ui")
        ui_right = Entity("gfx/ui_right.png").place("static_ui")

        ## TODO: investigate the off-by-one errors in pygext
        ui_left.bottom = 601
        ui_bottom.bottom = 601
        ui_right.bottom = 601

        ui_left.left = 1
        ui_bottom.left = ui_left.right
        ui_right.right = 801

        self.next = pylons.random_pylon()
        next = Entity(self.next.gfx)
        next.place("ui")
        next.scale = 0.65
        next.bottom = 600-15
        next.centerx = 800-45
        self.next_gfx = next

        act = ColorFade((255,255,255,255), secs=0.5)
        for l in "static_ui", "ui":
            for e in self[l]:
                e.color = (255,255,255,0)
                e.do(act)


    def random_next(self):
        self.next = pylons.random_pylon()
        self.init_next_gfx()
        
    def init_next_gfx(self):
        self.next_gfx.shape = self.next.gfx


    def show_intro(self):
        game = self.game
        font = game.font
        
        e = TextEntity(font, "Level %i" % game.level, scale=2)
        e2 = TextEntity(font, "GOAL: Electrify these items in order")
        e22 = TextEntity(font, "%i tiles\ntarget object" % self.level.goal)
        e3 = TextEntity(font, "Press mouse button to begin", scale=0.7)
        e4 = Entity("gfx/pylontarget.png").set(scale=0.5)

        e.place("intro").set(left=100, top=100)
        e2.place("intro").set(top=e.bottom+50, left=e.left)
        e22.place("intro").set(top=e2.bottom+20, centerx=e2.centerx)
        e3.place("intro").set(top=e22.bottom+50, left=e22.left)
        e4.place("intro").set(left=e22.right+20, bottom=e22.bottom)

        act = ColorFade((255,255,255,255), secs=1)
        for e in self["intro"]:
            e.color = (255,255,255,0)
            act.do(e)

    def state_intro_handle_mousebuttondown(self, ev):
        act = ColorFade((255,255,255,0), secs=0.5) + Delete

        for e in self["intro"]:
            e.do(act)

        act = Delay(0.5) + CallFunc(self.start_level)
        act.do()

    def start_level(self):
        self.state = "game"
        self.tilemap.drop_pieces()
        self.start_timer()
        self["intro"].clear()

    def stop_timers(self):
        for t in self.timers:
            t.abort_actions()

    def timer_end(self):
        self.tilemap.elec_callback = self.elec_end
        self.tilemap.electrify(10,10)
        self.state = "electric"

    def elec_end(self):
        self.state = "levelend"
        self.fader.hidden = False
        self.tilemap.remove_pieces()
        e = self.tilemap.count_electrified()
        self.level.done = e
        success = self.tilemap.target_hit
        self.level.success = success

        game = self.game

        tilebonus = e*100
        lines = [
            "Electrify goal: %i" % self.level.goal,
            "Tiles electrified: %i" % e,
            "Target electrified: %s" % (["No", "Yes"][success]),
            "Tile bonus: %i" % tilebonus,
            ]
        self.update_score(tilebonus)

        if e >= self.level.goal and success:
            lines.append("LEVEL CLEAR!")
            if len(self.timers) > 1:
                timebonus = (len(self.timers)-1) * 100
                lines.append("Time bonus: %i" % timebonus)
                self.update_score(timebonus)
            levelbonus = game.level * 1000
            lines.append("Level bonus: %i" % levelbonus)
            self.update_score(levelbonus)


        font = self.game.font
        y = 100        
        for i,l in enumerate(lines):
            e = TextEntity(font, l)
            e.place("extro").set(top=y, left=150+i*10)
            y = e.bottom
            e.color = (255,255,255,0)
            e.do(Delay(i*0.4) + ColorFade((255,255,255,255), secs=0.5))

        txt = TextEntity(font, "Click to continue", scale=0.7)
        txt.place("extro").set(centerx=400, bottom=550, color=(255,255,255,0))
        txt.do(Delay(i*0.4) + ColorFade((255,255,255,255), secs=0.5))

    def state_levelend_handle_mousebuttondown(self, ev):
        self.fader.hidden = True
        self["extro"].clear()
        if self.level.done >= self.level.goal and self.level.success:
            self.next_level()
        else:
            game = self.game
##            self.music_loop_src.Stop()
            game.music_src.Sound = pySonic.FileStream("sfx/cutemenu.mp3")
            game.music_src.LoopMode = pySonic.Constants.FSOUND_LOOP_NORMAL
            game.music_src.Play()
            director.set_scene(TitleScene, game)

    def tick(self):
        self.game.bg.tick()
        
        if director.ticker.realtick:
            
            self.check_scroll()
            tm = self.tilemap

            if self.state == "game":
                mx,my = pygame.mouse.get_pos()
                x,y = tm.picktile(mx,my)
                tm.cursx = x
                tm.cursy = y
            else:
                tm.cursx = -1
                tm.cursy = -1


    def handle_keydown(self, ev):
        if ev.key == K_ESCAPE:
            director.quit()

    def state_game_handle_mousebuttondown(self, ev):
        x,y = ev.pos
        tm = self.tilemap
        rx,ry = tm.picktile(x,y)
        t = tm.tiles.get((rx,ry))
        if ev.button == 1:
            if t and not t.object:
                tm.create(rx,ry, type=self.next)
                self.random_next()
        elif ev.button == 3:
            if rx == 10 and ry == 10:
                self.stop_timers()
                self.timer_end()
                #tm.electrify(10,10)
##            if t and t.object:
##                tm.lightning(rx,ry,vert=(ev.button == 2))
        elif ev.button == 2:
            self.zoomlevel = 0
            tm.zoom = 1.0
        elif ev.button == 4:
            self.zoomlevel -= 1
            tm.zoom = 0.8 ** self.zoomlevel
        elif ev.button == 5:
            self.zoomlevel += 1
            tm.zoom = 0.8 ** self.zoomlevel
        else:
            print ev.button

    def check_scroll(self):
        spd = 10
        k = pygame.key.get_pressed()
        sx = 0
        sy = 0
        if k[K_DOWN]:
            sy = spd
        elif k[K_UP]:
            sy = -spd
        elif k[K_LEFT]:
            sx = -spd
        elif k[K_RIGHT]:
            sx = spd
        self.tilemap.set_scroll(sx,sy)


def intro_movie():
    pygame.display.init()
    d = pygame.display.set_mode((800,600),0 , 16)
    s = pygame.Surface((800,600), 0, 32)
    m = pygame.movie.Movie("gfx/intro.mpg")
    print m
    print m.has_video()
    m.play()
    m.set_display(s, (0,0,800,600))
    pygame.event.set_allowed(None)
    pygame.event.set_allowed([KEYDOWN, MOUSEBUTTONDOWN, QUIT])
    while m.get_busy():
        if pygame.event.get():
            break
        d.blit(s, (0,0))
        pygame.display.update()
        time.sleep(0.1)
    del s
    m.stop()
    #pygame.quit()

def run():
    #intro_movie()
    init((800,600),(800,600), title="Pylonoid")
    game = Game()
    director.run(InitScene, game)
