
import pygame
import random, bisect
from pygext.gl.resources import resources
from pygext.render.stencils import alpha_stencil

PYLONS = ["21", "22", "23", "24", "i1", "i2",
          "t1", "t2", "t3", "t4",
          "v1", "v2", "v3", "v4", "x", "target"]

directions = [
    [(1,0)],
    [(0,1)],
    [(-1,0)],
    [(0,-1)],
    [(1,0),(-1,0)],
    [(0,1),(0,-1)],
    [(-1,0),(0,1),(0,-1)],
    [(-1,0),(1,0),(0,-1)],
    [(0,-1),(0,1),(1,0)],
    [(-1,0),(1,0),(0,1)],
    [(-1,0),(0,-1)],
    [(0,-1),(1,0)],
    [(1,0),(0,1)],
    [(-1,0),(0,1)],
    [(-1,0),(1,0),(0,1),(0,-1)],
    [],
    ]
    

types = []

i = iter(range(20))

probabilities = [
    (100, i.next()),
    (100, i.next()),
    (100, i.next()),
    (100, i.next()),
    (60, i.next()),
    (60, i.next()),
    (30, i.next()),
    (30, i.next()),
    (30, i.next()),
    (30, i.next()),
    (100, i.next()),
    (100, i.next()),
    (100, i.next()),
    (100, i.next()),
    #(15, 10),
    ]

namemap = {}

def init_pylons():
    for i,P in enumerate(PYLONS):
        t = PylonType(P, directions[i])
        types.append(t)
        namemap[P] = t

def weighted_choice(itemlist):
    if not itemlist:
        return None
    tresholds = []
    t = 0
    sum = 0
    for weight,item in itemlist:
        sum += weight
        
    for weight,item in itemlist:
        t += (weight/float(sum))
        tresholds.append(t)

    r = random.random()
    item = bisect.bisect_left(tresholds, r)
    return itemlist[item][1]

def random_pylon():
    i = weighted_choice(probabilities)
    return types[i]

class PylonType(object):

    def __init__(self, code, directions):
        self.code = code
        self.load_gfx()
        self.directions = directions

    def load_gfx(self):
        img = "gfx/pylon%s.png" % self.code
        self.gfx = resources.get_bitmap(img).set_hotspot(0.5, 1.05).scale(0.6)

        s = pygame.image.load(img)
        s2 = alpha_stencil(s)

        self.gfx_stencil = resources.init_bitmap("pylon%s_stencil" % self.code, s2).set_hotspot(0.5, 1.05).scale(0.6)
        self.gfx_shadow = resources.init_bitmap("pylon%s_shadow" % self.code, s2).set_hotspot(0.5, 1.05).scale(0.6,0.35).skewx(-0.55).translate(-5,-5)
