
from pygext.gl.all import *
#from OpenGL.GL import glPushMatrix, glPopMatrix, glTranslatef
from OpenGL.GL import *
import random
import pygame
import time
import pySonic

import pylons

try:
    Set = set
except NameError:
    from sets import Set

class TileMap(EntityNode):

    def __init__(self, size=10):
        EntityNode.__init__(self)

        self.tileimage = resources.get_bitmap("gfx/newtile3.png")
        self.tilesize = self.tileimage.size
        self.viewpos = ViewPosEntity(0,20)
        self.scroller = EntityMover(self.viewpos)

        self.cursx = -1
        self.cursy = -1
        c = self.cursor = Entity("tile_stencil")
        c.color = (255,255,255,100)
        #ColorFade((255,255,255,100), secs=0.5, mode=PingPongMode).do(c)
        

        self.tiles = {}
        #sfx = self.sfx_electric = pygame.mixer.Sound("sfx/electric.wav")
        #sfx.set_volume(0.5)
        self.sfx_electric = pySonic.FileSample("sfx/electric.wav")
        self.last_sfx = 0

##        for x in range(0,6):
##            for y in range(0,6):
##                if random.random() < 0.8:
##                    t = self.tiles[(x,y)] = Tile()
        self.generate_map(size)

        self.height = 16/80.0
        ih = 1-self.height

        sx,sy = self.tilesize
        h = self.height * sx
        sy *= ih
        ssx,ssy = sx/2.0, sy/2.0
        self.ssx = ssx
        self.ssy = ssy
        viewpoly = poly([
            (0,0),(5.0*ssx,5.0*ssy),
            (0.0*ssx,5.0*ssy+5.0*ssy),(-5.0*ssx,5.0*ssy),
            ]).scale(0.95).translate(ssx,sy*1.6)
        shader = rect(-sx*2.5,-ssy*1, sx*8,sy*8)
        shader = polycut(shader, viewpoly)
        #shader = viewpoly
        shader.fillcolor((0,0,0))
        #shader.fillcolor((255,0,0))
        self.shader = shader
        self.zoom = 1.0

        self.target = None
        self.target_hit = False
        self.elec_actions = Set()
        self.elec_callback = None

        e = Entity("tile_stencil")
        e.color = (255,255,255,20)
        act = ColorFade((200,255,255,50), secs=0.5, mode=PingPongMode)
        e.do(act)
        self.elec_stencil = e

    def raise_pieces(self):
        for t in self.tiles.itervalues():
            height = random.randrange(1000,1500)
            t.y = height

    def drop_pieces(self):
        for t in self.tiles.itervalues():
            height = t.y
            time = float(height)/random.randrange(1000,1500)
            MoveDelta(0, -height, secs=time).do(t)

    def remove_pieces(self):
        for t in self.tiles.itervalues():
            delay = random.random()
            t.do(Delay(delay) + MoveDelta(0, -1500, secs=2))

    def count_electrified(self):
        result = 0
        for t in self.tiles.itervalues():
            if t.object and t.object.electrified:
                result += 1
        return result

    def generate_map(self, size):
        last = None
        for x in range(8,13):
            for y in range(8,13):
                last = self._itgen(x, y, size) or last
        last.object = TargetPylon()
        self.tiles[10,10].object = MainPylon()
        self.indices = self.tiles.keys()
        self.indices.sort(lambda (a,b),(c,d): cmp((b,a),(d,c)))

    def _itgen(self, x, y, n):
        if n == 0 or self.tiles.get((x,y)) or abs(x-10) > 10 or abs(y-10) > 10:
            return
        t = Tile()
        self.tiles[x,y] = t
        for i in range(2):
            dx,dy = random.choice(pylons.namemap["x"].directions)
            t = self._itgen(x+dx,y+dy, n-1) or t
        return t

    def enter(self):
        self.scroller.tick()
        ssx,ssy = self.ssx,self.ssy
        x,y = self.viewpos.x,self.viewpos.y
        glPushMatrix()
        z = self.zoom
        if z != 1.0:
            glScalef(z,z,z)
        #glTranslatef(-(x-y)*ssx,-(y+x)*ssy,0)
        glTranslatef(-x*ssx,-y*ssy,0)
        vsize = 11
####        for y in range(0,10):
####            for x in range(0,10):
####                t = self.tiles.get((x,y))
##                if t:
##                    self.tileimage._draw()
##                    if x == self.cursx and y == self.cursy:
##                        # highlight
##                        c = self.cursor
##                        c.enter(); c.exit()
##                    if t.object:
##                        t.object.traverse()
##                glTranslatef(ssx,ssy,0)
##            glTranslatef(ssx*-vsize, ssy*-(vsize-2), 0)
        for x,y in self.indices:
            tx = (x-y)*ssx
            ty = (y+x)*ssy
            t = self.tiles.get((x,y))
            if t:
                hy = t.y
                glTranslatef(tx,ty-hy,0)
                self.tileimage._draw()
                #t.traverse()
                if x == self.cursx and y == self.cursy:
                    # highlight
                    c = self.cursor
                    c.enter()
                o = t.object
                if o:
                    if o.electrified:
                        self.elec_stencil.enter()
                    o.traverse()
                glTranslatef(-tx,-ty+hy,0)
        glPopMatrix()
        #self.shader._draw()

    def create(self, x, y, type):
        p = Pylon(type)
        p.color = (0,0,0,0)
        def show_pylon():
            p.color = None
            ColorFade((0,0,0,100), secs=0.6).do(p.shadow)
        self.tiles[x,y].object = p
        stencil = Entity(type.gfx_stencil)
        stencil.color = (255,255,225,0)
        act = ColorFade((255,255,255,255), secs=0.3) \
              + CallFunc(show_pylon) \
              + ColorFade((255,255,255,0), secs=0.6) \
              + Delete
        stencil.do(act)
        stencil.attach_to(p)

    def draw_lightning(self, x, y, vert=False):
        p = self.tiles[x,y].object
        if not p:
            return        
        l = Entity()
        if vert:
            l.scale = (-1,1)
        l.y = -111
        #l.y = -80
        #l.x = 30
        l.attach_to(p)
        l.color = (255,255,255,0)
        #p.lightning = l
        def remove_lightning():
            p.lightning = None
        anim = resources.get_anim("lightning")
        act = Animate(anim, secs=0.2, mode="repeat").limit(time=1.5) #+ CallFunc(remove_lightning)        
        l.do(act + Delete)
        l.do(ColorFade((255,255,255,255), secs=0.1) +
             Delay(1.1) +
             ColorFade((0,0,0,0), secs=0.3))

    def electrify(self, x, y):
        if self.target_hit:
            return
        
        p = self.tiles[x,y].object
        if p.electrified:
            return
              
        if type(p) is TargetPylon:
            self.target_hit = True
            return

        p.electrified = True

        #def unelectrify():
        #    p.electrified = False
        #act = Delay(2) + CallFunc(unelectrify)
        #act.do()

        onward = False
        for dx,dy in p.type.directions:
            tx = x + dx
            ty = y + dy
            tile = self.tiles.get((tx,ty))
            if tile and tile.object and not tile.object.electrified:
                o = tile.object
                for ox,oy in o.type.directions:
                    if ox == -dx and oy == -dy:
                        break
                else:
                    # draw l
                    if dx == 0:
                        vert = True
                    else:
                        vert = False
                    if dx < 0 or dy < 0:
                        self.draw_lightning(x, y, vert)
                    else:
                        self.draw_lightning(tx, ty, vert)
                    onward = True
                        
                    # set timer for electrify
                    act = Delay(1.1) + CallFunc(self.electrify, tx, ty)
                    act.chain(CallFunc(self._remove_elec, act))
                    self.elec_actions.add(act)
                    act.do()

        if onward:
            t = time.time()
            if t-self.last_sfx >= 1.0:
                #self.sfx_electric.play(0, 1200)
                self.last_sfx = t
                src = pySonic.Source()
                src.Sound = self.sfx_electric
                src.Play()
                def stop_sound():
                    src.Stop()
                act = Delay(1.2)+CallFunc(stop_sound)
                act.do()
        
        
    def set_scroll(self, vx, vy):
        self.scroller.vx = vx
        self.scroller.vy = vy

    def _remove_elec(self, act):
        self.elec_actions.discard(act)
        if not self.elec_actions and self.elec_callback:
            self.elec_callback()

    def picktile(self, mx, my):
        ssx,ssy = self.ssx,self.ssy
        z = self.zoom
        mx -= self.x
        my -= self.y
        mx += self.viewpos.x*ssx*z
        my += self.viewpos.y*ssy*z
        ty = (my/ssy-mx/ssx) / 2.0
        tx = ty + mx/ssx
        return int(tx/self.zoom+0.65),int(ty/self.zoom+0.65)
##        return int(tx/self.zoom+0.65+self.viewpos.x),int(ty/self.zoom+0.65+self.viewpos.y)
        

class Pylon(EntityNode):   
    def __init__(self, type):
        EntityNode.__init__(self, type.gfx)
        self.electrified = False
        self.type = type
        s = self.shadow = Entity(type.gfx_shadow)
        s.color = (0,0,0,0)

    def enter(self):
        s = self.shadow
        s.enter()
        EntityNode.enter(self)
       

class MainPylon(Pylon):
    def __init__(self):
        Pylon.__init__(self, pylons.namemap["x"])
        stencil = Entity(self.type.gfx_stencil)
        stencil.attach_to(self)
        stencil.color = (0,255,255,10)
        act = ColorFade((0,255,255,70), secs=0.15) \
              + ColorFade((0,255,255,10), secs=0.2) \
              + Delay(0.58)
        act = Repeat(act)
        stencil.do(act)
        #ColorFade((0,255,255,70), secs=0.7, mode=PingPongMode).do(stencil)

class TargetPylon(Pylon):
    def __init__(self):
        Pylon.__init__(self, pylons.namemap["target"])

class Tile(Entity):
    def __init__(self):
        Entity.__init__(self)
        self.object = None

class ViewPosEntity(Entity):
    def __init__(self, x, y):
        Entity.__init__(self)
        self.x = x
        self.y = y
